/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 8-1 - goodbye.c - Multiple callback lists and call_data
 *  
 *  HDR2: Chapter 8, page 172
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

/*** Include required header files **********************************/

#include <X11/Intrinsic.h>                   /* The Intrinsics      */
#include <Xm/Xm.h>                           /* Motif symbols       */
#include <Xm/RowColumn.h>                    /* XmRowColumn         */
#include <Xm/PushB.h>                        /* XmPushButton        */
#include <Xm/Text.h>                         /* XmText widget       */

/*** Declare global variables ***************************************/

XtAppContext  context;                       /* Application context */
Widget     toplevel, manager,                /* Widget tree         */
       exit_button, text_area;

/*** Forward declare callback routines ******************************/

void Arm();                                  /* callback routine    */
void Disarm();                               /* callback routine    */
void Activate();                             /* callback routine    */
void Merged();                               /* Combines all funcs  */
                                             /* 3 callbacks in one  */

/*** Start of Main  *************************************************/

main(argc,argv)
int argc;
char **argv;
{

/***  Init. Toolkit, Appl. Context, Application Shell   *************/

 toplevel = XtAppInitialize(&context, "Goodbye", 
     NULL, 0, &argc, argv, NULL, NULL, 0);

/***  Create widget tree and bind callbacks *************************/

 manager = XtCreateManagedWidget("manager",          /* XmRowColumn */
     xmRowColumnWidgetClass, toplevel, NULL, 0);

 text_area = XtCreateManagedWidget("textArea",       /* XmText child*/
     xmTextWidgetClass, manager, NULL, 0);

 exit_button = XtCreateManagedWidget("exitButton",   /* XmPushButton*/
     xmPushButtonWidgetClass, manager, NULL, 0);
 XtAddCallback(exit_button, XmNactivateCallback, Merged, NULL);
 XtAddCallback(exit_button, XmNarmCallback, Merged, NULL);
 XtAddCallback(exit_button, XmNdisarmCallback, Merged, NULL);

 XtRealizeWidget(toplevel);                          /* Realize all */
 XtAppMainLoop(context);                             /* Start Loop  */
}

/***** Callback Routines *********************/

void Arm(w, client_data, call_data)              /* Called if button*/
Widget  w;                                       /* is "armed"      */ 
XtPointer client_data, call_data;
{ 
printf("Arm\n");
}

/********************************************************************/

void Activate(w, client_data, call_data)         /* Called if button*/
Widget  w;                                       /* is "activated"  */
XtPointer client_data, call_data;
{
printf("Activate\n");
}

/********************************************************************/

void Disarm(w, client_data, call_data)           /* Called if button*/
Widget  w;                                       /* is "disarmed"   */
XtPointer client_data, call_data;
{
printf("disarm\n");
}

/********************************************************************/

void Merged(w, client_data, call_data)           /* Merged callback */
Widget  w;                                       /* uses call_data  */
XtPointer client_data, call_data;                /* to perform all  */
{                                                /* functions       */
 XmAnyCallbackStruct *cdata =                    /* Cast call_data  */
     (XmAnyCallbackStruct*)call_data;
 switch (cdata->reason){               /* switch on "reason" field  */
  case XmCR_ARM: {                               /* armed -         */
     XtVaSetValues(text_area,                    /* complain        */
    XmNvalue, "Please don't exit!", NULL);
     break;
  }
  case XmCR_DISARM: {                            /* disarmed -      */
     XtVaSetValues(text_area,                    /* express relief  */
    XmNvalue, "Thanks for staying!", NULL);
     break;
  }
  case XmCR_ACTIVATE: {                          /* activated -     */
     exit();                                     /* go away         */
  }
  default: return;                               /* any other reason*/
 }
}

