/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 11-1 - popup.c - Using a pop-up window to get data
 *  
 *  HDR2: Chapter 11, page 327
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

/*****  Include Required Header Files    *****************************/

#include <X11/Intrinsic.h>                 /* The Intrinsics         */
#include <Xm/Label.h>                      /* XmLabel widget         */
#include <Xm/Text.h>                       /* XmText widget          */
#include <Xm/PushB.h>                      /* XmPushButton widget    */
#include <Xm/RowColumn.h>                  /* XmRowColumn widget     */

/*****  Declare global variables, callbacks  *************************/

Widget toplevel, manager, popup_btn, exit_btn;
Widget dialog_shell, dialog_box, dialog_text, dialog_label, dialog_btn;
void CreateDialogBox();            /* Function - create dialog box   */
void PopupDialogBox();             /* Callback - pop up dialog box   */
void PopdownDialogBox();           /* Callback - pop down dialog box */
void ExitCallback();               /* Callback - exit                */

/*****  Start of main()   ********************************************/

main(argc,argv)
int argc;
char **argv;
{
  XtAppContext  context;

/*****  Init. Toolkit, Appl. Context, Application Shell  *************/

  toplevel = XtAppInitialize( &context, "Popup",
      NULL, 0, &argc, argv, NULL, NULL, 0 );

/*****  Create widget tree, bind callbacks  **************************/

  manager = XtCreateManagedWidget("manager",
      xmRowColumnWidgetClass, toplevel, NULL, 0);

  popup_btn = XtVaCreateManagedWidget("popupBtn",
       xmPushButtonWidgetClass, manager, NULL, 0);
  XtAddCallback(popup_btn, XmNactivateCallback, PopupDialogBox, NULL);

  exit_btn = XtCreateManagedWidget("exitBtn", 
       xmPushButtonWidgetClass, manager, NULL, 0);
  XtAddCallback(exit_btn, XmNactivateCallback, ExitCallback, NULL);

  CreateDialogBox();
  XtRealizeWidget(toplevel);
  XtAppMainLoop(context);                 /* Can't get past here !!! */
}

/****** Callback Routines ********************************************/

void CreateDialogBox()
{
 dialog_shell = XtCreatePopupShell("popupShell",
     transientShellWidgetClass, toplevel, NULL, 0);

 dialog_box = XtCreateManagedWidget("dialogBox", 
     xmRowColumnWidgetClass, dialog_shell, NULL, 0);

 dialog_label = XtCreateManagedWidget("dialogLabel", 
     xmLabelWidgetClass, dialog_box, NULL, 0);

 dialog_text = XtCreateManagedWidget("dialogText", 
     xmTextWidgetClass, dialog_box, NULL, 0);

 dialog_btn = XtCreateManagedWidget("dialogBtn", 
     xmPushButtonWidgetClass, dialog_box, NULL, 0);

 XtAddCallback(dialog_btn, XmNactivateCallback, 
     PopdownDialogBox, NULL);
 XtAddCallback(dialog_text, XmNactivateCallback, 
     PopdownDialogBox, NULL);

 XtRealizeWidget(dialog_shell);
}

/*********************************************************************/

void PopupDialogBox(w, client, call)
Widget w;
XtPointer client, call;
{
  /* Calculate position of dialog box as offset from app. shell */
  /* then pop it up */

  Position x, y, root_x, root_y;
  XtVaGetValues(toplevel, XmNx, &x, XmNy, &y, NULL);
  XtTranslateCoords(toplevel, x, y, &root_x, &root_y);
  XtVaSetValues(dialog_shell, XmNx, root_x+50, XmNy, root_y+50, NULL);
  XtPopup(dialog_shell, XtGrabExclusive);
}

/*********************************************************************/

void PopdownDialogBox(w, client, call)
Widget w;
XtPointer client, call;
{
  /* Retrieve input from text widget and set title bar */
  String string;
  XtVaGetValues(dialog_text, XmNvalue, &string, NULL);
  if (strlen(string)>0) {
    XtVaSetValues(toplevel, XmNtitle, string, NULL);
    XFree(string);
  }
  /* Pop down dialog box */
  XtPopdown(dialog_shell);
}

/*********************************************************************/

void ExitCallback(w, client, call)
Widget w;
XtPointer client, call;
{
 exit(1);
}

